%% Inteligentni sistemi - Cikel 3: Linearna regresija

%  Navodila
%  ------------
% 
%  Ta skripta vsebuje kodo, ki vam bo v pomo pri opravljanju vaje. 
%  Da boste uspeno opravili vajo boste morali dopolniti naslednje 
%  funkcije: 
%
%     plotData.m
%     gradientDescent.m
%     computeCost.m
%     gradientDescentMulti.m
%     computeCostMulti.m
%     featureNormalize.m
%     normalEqn.m
%
%  V datotekah, ki niso omenjene na zgornjem seznamu, ni potrebno 
%  spreminjati kode.
%
% x se nanaa na tevilo prebivalcev v 10.000-ih 
% y se nanaa na dobiek v 10,000-ih
%

%% Inicializacija
clear ; close all; clc

%% ======================= Vizualizacija =======================
fprintf('Vizualizacija podatkov ...\n')
data = load('ex3data1.txt');
X = data(:, 1); y = data(:, 2);
m = length(y); % tevilo unih primerov

% Izris
% opomba: dopolniti morate kodo v datoteki plotData.m
plotData(X, y);

fprintf('Program je prekinjen. Pritisni ENTER za nadaljevanje.\n');
pause;

%% =================== Gradientni spust ===================
fprintf('Izvajanje gradientnega spusta ...\n')

X = [ones(m, 1), data(:,1)]; % dodaj stolpec enic na zaetek X
theta = zeros(2, 1); % inicializacija parametrov

% Nastavitve za gradientno metodo
iteracije = 1500;
alpha = 0.01;

% izraun in prikaz zaetnega stroka
computeCost(X, y, theta)

% poeni gradientni spust
theta = gradientDescent(X, y, theta, alpha, iteracije);

% izpis parametrov theta na zaslon
fprintf('Parametri theta, ki smo jih nali z gradientnim spustom: ');
fprintf('%f %f \n', theta(1), theta(2));

% Izris linearnega prilagajanja (fitiranja)
hold on; % ohrani prejnji graf
plot(X(:,2), X*theta, '-')
legend('Uni podatki', 'Linearna regresija')
hold off % ne dodajaj ve dodatnih grafov na sliko 

% Napoved vrednosti dobika za tevilo prebivalcev 35,000 in 70,000
predict1 = [1, 3.5] *theta;
fprintf('Pri tevilu prebivalcev = 35,000, lahko napovemo dobiek %f\n',...
    predict1*10000);
predict2 = [1, 7] * theta;
fprintf('Pri tevilu prebivalcev = 70,000, lahko napovemo dobiek %f\n',...
    predict2*10000);

fprintf('Program je prekinjen. Pritisni ENTER za nadaljevanje.\n');
pause;

%% ============= Visualizacija J(theta_0, theta_1) =============
fprintf('Vizualizacija J(theta_0, theta_1) ...\n')

% Mrea, nad katero bomo raunali J(theta_0, theta_1) 
theta0_vals = linspace(-10, 10, 100);
theta1_vals = linspace(-1, 4, 100);

% inicializacija J_vals na matriko samih niel
J_vals = zeros(length(theta0_vals), length(theta1_vals));

% Polnjenje matrike J_vals
for i = 1:length(theta0_vals)
    for j = 1:length(theta1_vals)
	  t = [theta0_vals(i); theta1_vals(j)];    
	  J_vals(i,j) = computeCost(X, y, t);
    end
end


% zaradi delovanja ukaza surf na mrene grafe moramo transponirati matriko
% J_vals pred klicanjem tega ukaza, sicer bodo na grafu obrnjene osi
J_vals = J_vals';
% Surface plot
figure;
surf(theta0_vals, theta1_vals, J_vals)
xlabel('\theta_0'); ylabel('\theta_1');

% Contour plot
figure;
% Izris matrike J_vals v obliki 15 contour grafov na logaritmini skali med 0.01 in 100
contour(theta0_vals, theta1_vals, J_vals, logspace(-2, 3, 20))
xlabel('\theta_0'); ylabel('\theta_1');
hold on;
plot(theta(1), theta(2), 'rx', 'MarkerSize', 10, 'LineWidth', 2);
